import datetime
from dateutil.tz import tzutc

from .fields import * #: noqa

class TestDateTime():
    def test_dt_string(self):
        result = DateTime(dt_format="%m/%d/%Y").deserialize("03/15/2019")
        assert result == datetime.datetime(2019, 3, 15)

    def test_isoformat(self):
        dt = datetime.datetime.now()
        result = DateTime().deserialize(dt.isoformat())
        assert result == dt

    def test_bad_isoformat(self):
        field = DateTime()
        result = field.deserialize("2017-04-18T04:00:00Z")
        assert result == datetime.datetime(2017, 4, 18, 4, 0, tzinfo=tzutc())

class TestDate():
    def test_dt_string(self):
        result = Date(dt_format="%m/%d/%Y").deserialize("03/15/2019")
        assert result == datetime.date(2019, 3, 15)

    def test_isoformat(self):
        dt = datetime.datetime.now().date()
        result = Date().deserialize(dt.isoformat())
        assert result == dt

    def test_bad_isoformat(self):
        field = Date()
        result = field.deserialize("2017-04-18T04:00:00Z")
        assert result == datetime.date(2017, 4, 18)


from yankee.base import schema

from .fields import List
from .key import JsonMixin


class Schema(JsonMixin, schema.Schema):
    pass


class PolymorphicSchema(JsonMixin, schema.PolymorphicSchema):
    pass

class RegexSchema(JsonMixin, schema.RegexSchema):
    pass


import lxml.etree as ET

def do_nothing(obj):
    return obj

class FastXPath(ET.XPath):
    def __init__(self, xpath, *args, many=False, **kwargs):
        self.many = many
        if many:
            super().__init__(xpath, *args, **kwargs)
        else:
            xpath = f"({xpath})[1]"
            super().__init__(xpath, *args, **kwargs)
    
    def __call__(self, *args, **kwargs):
        result = super().__call__(*args, **kwargs)
        if self.many:
            return result
        elif result:
            return result[0]
        else:
            return None

class XmlMixin(object):
    def make_accessor(self):
        namespaces = getattr(self.Meta, "namespaces", dict())
        if self.data_key is None:
            return do_nothing
        return FastXPath(self.data_key, many=self.many, namespaces=namespaces)

    def to_string(self, elem):
        if isinstance(elem, str):
            return elem
        return "".join(elem.itertext())

    def convert_groupdict(self, dictionary):
        root = ET.Element("root")
        for k, v in dictionary.items():
            subel = ET.SubElement(root, k)
            subel.text = v
        return root
